/*
 Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA
*/

/**
   \file
   \brief The macro is used to filter a data set based on ENTITY_ID and ENTITY ROLL in specified destination.

    \details
          If ENTITY_ID is a column of the table, only data pertaining to entity (group) structure
          (user selects which entity/group he wants to run) will be retrievable for the user.
          Certain filtering will be performed.
          1. if solo case, only retrieve data pertaining to that solo entity.
          2. if group case, retrieve the group structure first, and retrieve data pertaining
          to all the entities inside the group.
          
    \notes
          For the export function to work, entity table must have the following columns:
          ENTITY_ID GROUP_ID ENTITY_ROLE_CD and REPORTING_CURRENCY_CD. 

   <b> Identified Inputs </b>

   \param[in]  SRC_PATH                   The path of the SAS data set that will be filtered.

   \param[in]  SRC_FILE                   The name of the SAS data set that will be filtered (not including library name).

   \param[in]  OUT_PATH                   location of the output table after filtering.

   \param[in]  OUT_FILE                   output table name after filtering.

   \param[in]  FLOW_ENTITY_NM             flow entity table name(this contains information for entity_id, entity_role, 
                                          staging_path and flowinstance_key)

   \param[in]  FLOW_ENTITY_LOC            flow entity table location

   \param[in]  IN_ENTITY_TBL_NM           name of entity table

   \param[in]  INSTANCE_KEY               instance key mid-tier pass in for the specific action


    \ingroup CommonServer

    \author SAS Institute INC.
    \date 2017

 */
%macro irm_filter_data_by_flow(SRC_PATH=,
                             SRC_FILE=,
                             OUT_PATH=,
                             OUT_FILE=,
                             FLOW_ENTITY_NM=,
                             FLOW_ENTITY_LOC=,
                             IN_ENTITY_TBL_NM=,
                             INSTANCE_KEY=
                             );
                             
%rsk_trace(Entry);
   /* initialize entity_list macro for further use*/
   %local entity_list output_lib IN_ENTITY IN_ENTITY_ROLL STAG_LIB_PATH;
   %let entity_list=;
 
   libname INFO "&FLOW_ENTITY_LOC.";
   
   /* first read in the flow entity information for the specific instance key*/
   data work.flow_info;
      set INFO.&FLOW_ENTITY_NM;
      where flowinstance_key=&INSTANCE_KEY.;
      call symput('IN_ENTITY_ROLL', trim(entity_role));
      call symput('STAG_LIB_PATH', trim(staging_path));
   run;
   
   LIBNAME SRC_LIB "&SRC_PATH";
   %if "&OUT_PATH" ne "" %then %do;
         LIBNAME OUT_LIB "&OUT_PATH";
         %let output_lib=OUT_LIB;
   %end;
   %else %do;
         %let output_lib=work;
   %end;
   options validvarname = upcase;
   /* assign the STAGING library as STAGING.ENTITY table will be used*/
   LIBNAME STAGING "&STAG_LIB_PATH.";

 /* check if ENTITY_ID column is inside the table*/
   %if %rsk_varexist(SRC_LIB.&SRC_FILE.,ENTITY_ID) %then %do;
   
       	data all_entities;
            set STAGING.&IN_ENTITY_TBL_NM (keep=ENTITY_ID GROUP_ID ENTITY_ROLE_CD
            %if %rsk_varexist(STAGING.&IN_ENTITY_TBL_NM, SOLO_PARENT_FLG) %then %do;
            	SOLO_PARENT_FLG
            %end;
             REPORTING_CURRENCY_CD rename = (REPORTING_CURRENCY_CD=CURRENCY_CD));
         run;
         proc sql noprint;
             create table entity_structure
              as select * from all_entities
              where upcase(ENTITY_ID) in (select upcase(ENTITY_ID) from flow_info)
             ;
         quit;

          %if %rsk_attrn(entity_structure, nobs)=0 %then %do;
                %rsk_get_msg(key=irm_invalid_entity);            
          %end;
          	data sub_groups;
         		set flow_info;
			run;
          	proc datasets library=work nolist;
   				delete final_entities;
   			quit;
          
      /* check if this is a group/solo run*/
      %if %upcase(&IN_ENTITY_ROLL) ne SOLO %then %do;
      
  
         /* Construct source entity data set*/

        /* Loop through all layers under the parent entity to retrieve source entities */
         %do %while (%rsk_attrn(sub_groups, nobs) ne 0);
            %irm_get_child_parent_entity( IN_ENTITY_DS=all_entities,
                                          CHILD_LIST=sub_groups,
                                          IN_ENTITY_ROLE=GROUP,
                                          O_ENTITY_FINAL_DS=final_entities,
                                          O_ENTITY_SUB_GRP_DS=sub_groups);
         	%end;/* end of looping through all layers*/
            %if %rsk_attrn(final_entities, nobs) ne 0 %then %do;
                proc append base=work.entity_structure data=final_entities;
                run;
            %end;         
      %end;/* end group case*/
      %else %do;
         /*For solo case, check if SOLO_PARENT_FLG exist, if so switch to include non-legal entity*/
         %if %rsk_varexist(STAGING.&IN_ENTITY_TBL_NM, SOLO_PARENT_FLG) %then %do;
        /* Loop through all layers under the parent entity to retrieve source entities */
            %do %while (%rsk_attrn(sub_groups, nobs) ne 0);
               %irm_get_child_parent_entity( IN_ENTITY_DS=all_entities,
                                             CHILD_LIST=sub_groups,
                                             IN_ENTITY_ROLE=SOLO,
                                             O_ENTITY_FINAL_DS=final_entities,
                                             O_ENTITY_SUB_GRP_DS=sub_groups);

            %end;/* end of looping through all layers*/
            %if %rsk_attrn(final_entities, nobs) ne 0 %then %do;
                  proc append base=work.entity_structure data=final_entities;
                  run;
            %end;
            
         %end;/*end of check SOLO_PARENT_FLG*/
         %else %do;/*if SOLO_PARENT_FLG doesn't exist*/
            data work.entity_structure;
               set work.entity_structure;
            run;
         %end;
         
      
      %end;/*end of solo case*/
	 	proc sort data=entity_structure nodupkey;
	 	by ENTITY_ID;
	 	run;
     /* filter the data*/
      	proc sql noprint;
         	create table &output_lib..&OUT_FILE
         	as select * from SRC_LIB.&SRC_FILE
         	where upcase(ENTITY_ID) in (select distinct upcase(ENTITY_ID) from work.entity_structure);
      	quit;
      
   %end;
      /*end ENTITY_ID clause*/
   %else %do;
   
     data &output_lib..&OUT_FILE;
        set SRC_LIB.&SRC_FILE;
     run;
      
   %end;
   

   LIBNAME SRC_LIB CLEAR;
   %LET SIIRC=0;
   %rsk_trace(Exit);

%mend;
